# btsgtk/main_window.py
#  Copyright (C) 2008 Luca Bruno <lethalman88@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.

import gtk

from manager import Manager
from dialogs import InputStringDialog

class MainWindow (gtk.Window):
    def __init__ (self, application):
        gtk.Window.__init__ (self)
        self.application = application

        # Setup the main window widget
        self.setup_signals ()
        self.create_layout ()
        self.create_toolbar ()
        self.create_manager ()
        self.create_actions ()

        self.show_all ()

    def setup_signals (self):
        self.connect ('delete-event', self.application.quit)

    def create_layout (self):
        # The main layout to be used as child for the window
        self.layout = gtk.VBox ()
        self.add (self.layout)
        self.set_title ("Bug Tracking System")
        self.set_default_size (400, 400)

    def create_toolbar (self):
        toolbar = gtk.Toolbar ()
        
        button = gtk.ToolButton (gtk.STOCK_OPEN)
        button.set_tooltip_text ("Open a new bug")
        button.connect ('clicked', self.on_open_clicked)
        toolbar.insert (button, -1)

        button = gtk.ToolButton (gtk.STOCK_CLOSE)
        button.set_tooltip_text ("Close the current bug")
        button.connect ('clicked', self.on_close_clicked)
        toolbar.insert (button, -1)

        separator = gtk.SeparatorToolItem ()
        toolbar.insert (separator, -1)

        button = gtk.ToolButton (gtk.STOCK_EXECUTE)
        button.set_tooltip_text ("Execute all the pending operations")
        button.connect ('clicked', self.on_execute_clicked)
        toolbar.insert (button, -1)

        separator = gtk.SeparatorToolItem ()
        toolbar.insert (separator, -1)

        button = gtk.ToolButton (gtk.STOCK_QUIT)
        button.set_tooltip_text ("Quit the bts-gtk application")
        button.connect ('clicked', self.application.quit)
        toolbar.insert (button, -1)

        self.layout.pack_start (toolbar, expand=False)

    def create_manager (self):
        self.manager = Manager (self.application)
        self.layout.pack_start (self.manager)

    def create_actions (self):
        pass



    # Signals

    def on_open_clicked (self, button):
        dialog = InputStringDialog (self, "Please type the bug number")
        dialog.show_all ()
        dialog.connect ('response', self.on_open_input_response)

    def on_open_input_response (self, dialog, res):
        if res == gtk.RESPONSE_ACCEPT:
            number = int (dialog.get_value ())
            self.manager.open (number)
        dialog.destroy ()

    def on_close_clicked (self, button):
        self.manager.close_current ()

    def on_execute_clicked (self, button):
        self.manager.execute ()
