# btsgtk/status_icon.py
#  Copyright (C) 2008 Luca Bruno <lethalman88@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.

import gtk
from gtk import gdk

import btsop
from dialogs import *

class BugEntry (gtk.Entry):
    def __init__ (self, menu):
        gtk.Entry.__init__ (self)
        self.menu = menu
        self.set_text ("type the number")
        self.focus_in_id = self.connect ('focus-in-event', self.on_focus_in_event)
        self.show ()

    def get_number (self):
        try:
            number = int (self.get_text ())
            if not number:
                raise ValueError ()
            return number
        except:
            error_dialog (self.menu, "Bug number is not valid")

    def on_focus_in_event (self, *args):
        self.set_text ("")
        self.disconnect (self.focus_in_id)
        return False

class OpButton (gtk.Button):
    def __init__ (self, application, entry, btsop_class):
        gtk.Button.__init__ (self)
        self.application = application
        self.manager = application.get_manager ()
        self.entry = entry
        self.btsop_class = btsop_class

        self.set_label (btsop_class.name)
        self.show ()

        self.connect ('clicked', self.on_clicked)

    def on_clicked (self, button):
        number = self.entry.get_number ()
        if number:
            batch = self.manager.open (number)
            batch.add_operation (self.btsop_class (self.application))
            self.application.main_window.present ()

class Menu (gtk.Window):
    def __init__ (self, application):
        gtk.Window.__init__ (self)
        self.application = application
        self.fake_menu = gtk.Menu ()
        self.set_decorated (False)

        self.create_layout ()
        self.create_items ()

        self.connect ('button-press-event', self.on_button_press_event)
        self.connect ('focus-out-event', self.on_focus_out_event)
        self.connect ('map-event', self.on_map_event)

    def on_focus_out_event (self, *args):
        self.hide ()

    def on_button_press_event (self, menu, event):
        if event.x < 0 or event.y < 0:
            self.hide ()

    def create_layout (self):
        self.layout = gtk.VBox (spacing=12)
        self.layout.show ()
        self.add (self.layout)

    def create_items (self):
        frame = gtk.Frame ("Bug number")
        frame.show ()

        box = gtk.VBox (spacing=6)
        box.show ()
        self.entry = BugEntry (self)
        box.pack_start (self.entry, expand=False)

        button = gtk.Button (stock=gtk.STOCK_OPEN)
        button.connect ('clicked', self.on_open_clicked)
        button.show ()
        box.pack_start (button, expand=False)
        frame.add (box)
        self.layout.pack_start (frame, expand=False)
        
        frame = gtk.Frame ("Operations")
        frame.show ()
        box = gtk.VButtonBox ()
        for btsop_class in btsop.operations:
            box.pack_start (OpButton (self.application, self.entry, btsop_class))
        box.show ()
        frame.add (box)
        self.layout.pack_start (frame, expand=False)

        sep = gtk.VSeparator ()
        sep.show ()
        self.layout.pack_start (sep, expand=False)

        button = gtk.Button (stock=gtk.STOCK_QUIT)
        button.connect ('clicked', self.application.quit)
        button.show ()
        self.layout.pack_start (button, expand=False)

    def popup (self, parent_shell, parent_widget, func, button, time, status_icon):
        x, y, push = gtk.status_icon_position_menu (self.fake_menu, status_icon)
        self.move (x, y)
        self.set_screen (status_icon.get_screen ())
        self.show ()

    # Signals

    def on_map_event (self, *args):
        if gdk.pointer_grab (self.window, True, gdk.BUTTON_PRESS_MASK) != gdk.GRAB_SUCCESS:
            print 'asd'
            self.hide ()

    def on_open_clicked (self, button):
        number = self.entry.get_number ()
        if number:
            self.application.open_bug (number)
            self.application.main_window.present ()

class StatusIcon (gtk.StatusIcon):
    def __init__ (self, application):
        gtk.StatusIcon.__init__ (self)
        self.application = application
        self.set_from_file (self.application.logo)

        self.create_menu ()

        self.connect ('activate', self.on_activate)
        self.connect ('popup-menu', self.on_popup_menu)

    def create_menu (self):
        self.menu = Menu (self.application)

    def on_activate (self, *args):
        window = self.application.main_window
        if window.is_active ():
            window.hide ()
        else:
            window.present ()

    def on_popup_menu (self, status_icon, button, time):
        self.menu.popup (None, None, gtk.status_icon_position_menu, button, time, status_icon)
